//  SegParse.cpp : Defines the entry point for the DLL application.

/*
 * (C) Copyright IBM Corp. 2000
 */

/* Includes */

#ifdef WIN32
#include <windows.h>
#include "largeint.h"
#include <wchar.h>
#endif

#include <malloc.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <BipCpi.h>
#include <BipCni.h>
#include <cmqc.h>

#include "parsubs.h"
#include "trace.h"
#include "mtdstruc.h"
#include "metamem.h"
#include "metadata.h"
#include "SegParse.h"
#include "context.h"
#include "acordsub.h"
#include "mgmtmsg.h"
#include "common.h"

/**************************************************************/
/*                                                            */
/* Defines and global variables                               */
/*                                                            */
/**************************************************************/

#ifdef WIN32
#define ACORD_PARSER_CLASS	L"ACORD"
#define	ACORD_PARSER_NAME	L"AcordParserFactory"
#endif

#define ACORD_IGNORE_FILLER	"ACORD_IGNORE_FILLER"

static char copyright[] = "(C) Copyright IBM Corp, 2000";
static char Version[]=\
"@(#)SegParse V1.00 - MQSI V2.0 ACORD Message Parser  - Jim MacNair";

#ifdef _DEBUG
static char Level[]="SegParse.cpp V1.0 Debug version ("__DATE__" "__TIME__")";
#else
static char Level[]="SegParse.cpp V1.0 Release version ("__DATE__" "__TIME__")";
#endif

	int		parserStartHigh;	// Time parser was started (high word)
	int		parserStartLow;		// Time parser was started (low word)
	int     totalParseTime=0;	// Total parsing time in milliseconds
	int     totalWriteTime=0;	// Total time to build a buffer in milliseconds
	int     maxParseTime=0;		// Maximum time to parse a message
	int     minParseTime=0;		// Minimum time to parse a message
	int     maxWriteTime=0;		// Maximum time to build a buffer
	int     minWriteTime=0;		// Minimum time to build a buffer
	int     maxMessage=0;		// Largest input message processed
	int     minMessage=0;		// Smallest input message processed
	int     totalBytes=0;		// Number of bytes in all input messages processed
	int		totalUncomp=0;		// Number of bytes in all input after decompression
	int		totalWritten=0;		// Number of bytes written
	int		maxWriteLength=0;	// Largest message written
	int		minWriteLength=0;	// Smallest message written
	int		compressMsgs=0;		// Number of compressed messages processed
	int     writeCount=0;		// Number of cpiWriteBuffer calls processed
	int     messageCount=0;		// Number of messages processed    

	static int contextCount=0;	// Number of parser objects 

#ifdef _DEBUG
extern int debug;
extern int traceParser;
extern int traceWrite;
extern int traceModule;
extern int traceMgmt;
#endif

void saveMsgStats(PARSER_CONTEXT_ST* context)

{
	// save the statistics for this message
	if (context->totalParseTime > maxParseTime)
	{
		maxParseTime = context->totalParseTime;
	}

	if ((minParseTime == 0) || (context->totalParseTime < minParseTime))
	{
		minParseTime = context->totalParseTime;
	}

	// get the total number of bytes processed
	totalBytes += context->iLength;

	// check if this is the largest message processed
	if (context->iLength > maxMessage)
	{
		maxMessage = context->iLength;
	}

	// check if this is the smallest message
	if ((minMessage == 0) || (context->iLength < minMessage))
	{
		minMessage = context->iLength;
	}
}

/******************************************************************************/
/*                                                                            */
/* Routine to initialize the context area.                                    */
/*                                                                            */
/******************************************************************************/

void resetContext(PARSER_CONTEXT_ST* context)

{
	int envVal;
	char* envPtr;

	// initialize the context area to binary zeros
	memset(context, 0, sizeof(PARSER_CONTEXT_ST));

	/* check for ignore filler setting in environment */
	envPtr = getenv(ACORD_IGNORE_FILLER);
	if (envPtr != NULL)
	{
		envVal = atoi(envPtr);
		if (envVal > 0)
		{
			context->iFiller = 1;
		}
	}
}

/******************************************************************************/
/*                                                                            */
/* Routine to capture the starting time and save it in the context.           */
/*                                                                            */
/******************************************************************************/

void getStartTime(PARSER_CONTEXT_ST* context)

{
	SYSTEMTIME		curtime;
	FILETIME		filtime;

	// Get the current time
	GetSystemTime(&curtime);
	SystemTimeToFileTime(&curtime, &filtime);

	// Save the time in the context
	context->iStartTimeHigh = filtime.dwHighDateTime;
	context->iStartTimeLow = filtime.dwLowDateTime;
}

/******************************************************************************/
/*                                                                            */
/* Routine to calculate the difference between the current time and the       */
/* start time in the context and return the difference as an integer.         */
/*                                                                            */
/******************************************************************************/

int calculateTime(PARSER_CONTEXT_ST* context)

{
	int				time;
	SYSTEMTIME		curtime;
	FILETIME		filtime;
	LARGE_INTEGER	startTime;
	LARGE_INTEGER	endTime;
	LARGE_INTEGER	parseTime;

	// Get the current time
	GetSystemTime(&curtime);
	SystemTimeToFileTime(&curtime, &filtime);

	// Save the time in the work variable
	endTime.HighPart = filtime.dwHighDateTime;
	endTime.LowPart = filtime.dwLowDateTime;

	// Retrieve the starting time from the context
	startTime.HighPart = context->iStartTimeHigh;
	startTime.LowPart = context->iStartTimeLow;

	// Calculate the response time
	parseTime = LargeIntegerSubtract(endTime, startTime);

	time = (parseTime.LowPart / 10000);

	return time;
}

/******************************************************************************/
/*                                                                            */
/* Perform the initial processing steps to parse the message, including       */
/*  translation to Unicode and validating the message format.                 */
/*                                                                            */
/******************************************************************************/

void validateMessage(PARSER_CONTEXT_ST *pc)

{
	int		rc;
	int		extra;

	if (pc->iMsgType == 0)
	{
		// check if this message has any compression characters
		// Calculate how large the area must be
		extra = checkCompress(pc->iDataPtr, pc->iDataLen, pc->iCodePage);

		if (extra > 0)
		{
			// Count the number of compressed messages
			compressMsgs++;

			// Data is compressed - we must get a new area
			pc->iLength = pc->iDataLen + extra;
			pc->iBuffer = (CciChar *) malloc((pc->iLength) * sizeof(CciChar));
			deCompress(pc->iDataPtr, pc->iBuffer, pc->iDataLen, pc->iCodePage);
		}
		else
		{
			// we just need to acquire a data area and translate the incoming 
			// data to Unicode
			pc->iBuffer = (CciChar *) malloc(pc->iDataLen * sizeof(CciChar));
			cciMbsToUcs(&rc, pc->iDataPtr, pc->iBuffer, pc->iDataLen, pc->iCodePage);
		}

		/* Check if this is a valid ACORD AL3 message */
		checkMessageFormat(pc);

		// normal AL3 message - count it
		// keep a count of the number of input messages 
		messageCount++;

		// Number of bytes in all input messages processed
		totalBytes += pc->iDataLen;

		// Number of bytes in all input after decompression
		totalUncomp += pc->iLength;	
	}
}

/******************************************************************************/
/*                                                                            */
/* Plugin Parser Implementation Function:                                     */
/*                                                                            */
/******************************************************************************/

CciContext* cpiCreateContext(CciParser* parser)

{
	PARSER_CONTEXT_ST *pc;

#ifdef _DEBUG
	if (traceModule)
	{
		Log("\nPLUGIN: -> cpiCreateContext() parser=0x%x\n", parser);
	}
#endif

	/* Allocate a pointer to the local context */
	pc = (PARSER_CONTEXT_ST *) malloc(sizeof(PARSER_CONTEXT_ST));

	if (pc) 
	{
		/* Clear the context area */
		resetContext(pc);

		/* Keep count of the number of parser objects */
		contextCount++;
	}
	else 
	{
		/* Error: Unable to allocate memory */
#ifdef _DEBUG
		if (debug)
		{
			Log("Unable to allocate memory for Context for ACORD parser\n");
		}
#endif
	}

#ifdef _DEBUG
	if (traceModule)
	{
		Log("PLUGIN: <- cpiCreateContext(0x%x) count %d\n", pc, contextCount);
	}
#endif

	/* Return the pointer to the local context */
	return((CciContext*) pc);
}

/******************************************************************************/
/*                                                                            */
/* Plugin Parser Implementation Function:                                     */
/*                                                                            */
/* This routine will initialize or reinitialize the parser context.           */
/*                                                                            */
/******************************************************************************/

int cpiParseBuffer(CciParser* parser, CciContext* context, int offset)

{
	PARSER_CONTEXT_ST*	pc = (PARSER_CONTEXT_ST *) context;
	int					rc;
#ifdef _DEBUG
	SYSTEMTIME	curtime;
#endif

#ifdef _DEBUG
	// Get the current time
	GetSystemTime(&curtime);

	if (traceModule) 
	{
		Log("\nPLUGIN: -> cpiParseBuffer() %d/%d/%d %2.2d:%2.2d:%2.2d.%d \n",
			curtime.wYear,
			curtime.wMonth,
			curtime.wDay,
			curtime.wHour,
			curtime.wMinute,
			curtime.wSecond,
			curtime.wMilliseconds);

		Log(" parser (0x%x) context (0x%x) offset (%d)\n", 
			parser, context, offset);
	}
#endif

	/* Check if we have a left over data area from the last message          */
	/* This would have been allocated for the previous message */
	if ((pc->iBuffer != CCI_NULL_ADDR) && (pc->iBuffer != (unsigned short *) pc->iDataPtr))
	{
		free(pc->iBuffer);
	}

	/* Save the statistics from the previous message */
	if (pc->iBuffer != CCI_NULL_ADDR)
	{
		saveMsgStats(pc);
	}

	/* Initialize various data areas in the context */
	resetContext(pc);

	// Start the timer
	getStartTime(pc);

	/* Get a pointer to the message buffer and save the current offset */
	pc->iDataPtr = (char *) cpiBufferPointer(&rc, parser) + offset;

	/* Initialize the offset within the data input area */
	pc->iOffset = 0;

	/* Save size of the buffer */
	pc->iSize = cpiBufferSize(&rc, parser);

	/* Set the current element to the root element */
	pc->eParent = cpiRootElement(&rc, parser);

	/* remember the length of the body */
	pc->iDataLen = pc->iSize - offset;

	/* Initialize input data length */
	pc->iLength = pc->iDataLen;

	/* get the code page of the input data */
	pc->iCodePage = getCodePage(parser);

	/* make sure we have a sensible default code page */
	if (pc->iCodePage == 0)
	{
		pc->iCodePage = DEF_CODEPAGE;
	}

	/* get the numeric encoding for the message */
	pc->iEncoding = getEncoding(parser);

	/* make sure we have a proper encoding value */
	if (pc->iEncoding == 0)
	{
		/* set a sensible default */
		pc->iEncoding = DEF_ENCODING;
	}

	/* now, figure out if integers are PC or host format */
	if ((pc->iEncoding & MQENC_INTEGER_REVERSED) > 0)
	{
		pc->iEncodeInt = 1;
	}

	/* now, figure out if packed decimal are PC or host format */
	if ((pc->iEncoding & MQENC_DECIMAL_REVERSED) > 0)
	{
		pc->iEncodePD = 1;
	}

	/* get a pointer to the body element */
	pc->eBody =cpiRootElement(&rc, parser);

	/* check for a systems management message instead of a normal message */
	pc->iMsgType = checkForMgmtMsg(parser);

	if (pc->iMsgType == 1)
	{
		// we need to acquire a data area and translate the incoming 
		// data to Unicode
		pc->iBuffer = (CciChar *) malloc(pc->iDataLen * sizeof(CciChar));
		cciMbsToUcs(&rc, pc->iDataPtr, pc->iBuffer, pc->iDataLen, pc->iCodePage);

		// process any messages that should be processed immediately,
		// such as turning trace on or off, or flushing the cache
		processMgmtMsg(parser, pc);
	}

#ifdef _DEBUG
	if (traceModule) 
	{
		Log("PLUGIN: <- cpiParseBuffer() message size =%d\n", pc->iDataLen);
	}
#endif

	/* We will assume ownership of the remainder of the buffer */
	return(pc->iSize-offset);
}

/******************************************************************************/
/*                                                                            */
/* Plugin Parser Implementation Function:                                     */
/*                                                                            */
/* Parse first child will parse the message a group at a time, until          */
/* the first child complete flag (completePrevious) is set.                   */
/*                                                                            */
/******************************************************************************/

void cpiParseFirstChild(CciParser* parser, CciContext* context, CciElement* element)

{
	PARSER_CONTEXT_ST*	pc = (PARSER_CONTEXT_ST*) context;
	int					rc;
	int					time;
#ifdef _DEBUG
	CciChar	name[256];
	CciSize	length;
	CciBool	completePrev;
	CciBool completeNext;
#endif

#ifdef _DEBUG
	if (traceModule) 
	{
		memset(name, 0, sizeof(name));
		length = cpiElementName(&rc, element, name, sizeof(name));
		completePrev = cpiElementCompletePrevious(&rc, element);
		completeNext = cpiElementCompleteNext(&rc, element);
		Log("\nPLUGIN: -> cpiParseFirstChild() %S (0x%x) bits (%d %d)\n",
			name, element, completePrev, completeNext);
	}
#endif

	// Start the timer
	getStartTime(pc);

	// check for first time through
	if (pc->iOffset == 0)
	{
		validateMessage(pc);
	}

	// Check if we have a systems management message or a normal one
	if (pc->iMsgType == 0)
	{
		// Normal message, parse the next group
		while ((cpiElementCompletePrevious(&rc, element) != COMPLETE_BIT) &&
			   (pc->iOffset < pc->iLength))
		{
			parseNextSegment(parser, pc);
		}

		/* add to the total time */
		time = calculateTime(pc);
		totalParseTime += time;
		pc->totalParseTime += time;
	}
	else
	{
		// system management message
		// handle as special case
		// This will build a complete parse tree
		parseMgmtMsg(parser, pc);
	}

#ifdef _DEBUG
	if (traceModule) 
	{
		completePrev = cpiElementCompletePrevious(&rc, element);
		completeNext = cpiElementCompleteNext(&rc, element);
		Log("PLUGIN: <- cpiParseFirstChild() %S (0x%x) bits (%d %d)\n", 
			name, element, completePrev, completeNext);
	}
#endif

	return;
}

/******************************************************************************/
/*                                                                            */
/* Plugin Parser Implementation Function:                                     */
/*                                                                            */
/* Parse the message a group (segment) at a time until the completeNext bit   */
/* is set for this element, indicating that the last child of this element    */
/* has been parsed.                                                           */
/*                                                                            */
/******************************************************************************/

void cpiParseLastChild(CciParser* parser, CciContext* context, CciElement* element)

{
	PARSER_CONTEXT_ST*	pc = (PARSER_CONTEXT_ST*) context;
	int					rc;
	int					time;
#ifdef _DEBUG
	CciChar	name[256];
	CciSize	length;
	CciBool	completePrev;
	CciBool completeNext;
#endif

#ifdef _DEBUG
	if (traceModule) 
	{
		memset(name, 0, sizeof(name));
		length = cpiElementName(&rc, element, name, sizeof(name));
		completePrev = cpiElementCompletePrevious(&rc, element);
		completeNext = cpiElementCompleteNext(&rc, element);
		Log("\nPLUGIN: -> cpiParseLastChild() %S (0x%x) bits(%d %d)\n",
			name, element, completePrev, completeNext);
	}
#endif

	// Start the timer
	getStartTime(pc);

	// check for first time through
	if (pc->iOffset == 0)
	{
		validateMessage(pc);
	}

	// Check if we have a systems management message or a normal one
	if (pc->iMsgType == 0)
	{
		// Normal message, parse the next group until the element is completely parsed
		while ((cpiElementCompleteNext(&rc, element) != COMPLETE_BIT) &&
			   (pc->iOffset < pc->iLength))
		{
			parseNextSegment(parser, pc);
		}

		/* add to the total time */
		time = calculateTime(pc);
		totalParseTime += time;
		pc->totalParseTime += time;
	}
	else
	{
		// system management message
		// handle as special case
		// This will build a complete parse tree
		parseMgmtMsg(parser, pc);
	}

#ifdef _DEBUG
	if (traceModule) 
	{
		completePrev = cpiElementCompletePrevious(&rc, element);
		completeNext = cpiElementCompleteNext(&rc, element);
		Log("PLUGIN: <- cpiParseLastChild() %S (0x%x) bits (%d %d)\n", 
			name, element, completePrev, completeNext);
	}
#endif

  return;
}

/******************************************************************************/
/*                                                                            */
/* Plugin Parser Implementation Function:                                     */
/*                                                                            */
/******************************************************************************/

void cpiParseLeftSibling(CciParser* parser, CciContext* context, CciElement* element)

{
	PARSER_CONTEXT_ST*	pc = (PARSER_CONTEXT_ST*) context;
	int					rc;
	int					time;
#ifdef _DEBUG
	CciChar	name[256];
	CciSize	length;
	CciBool	completePrev;
	CciBool completeNext;
#endif

#ifdef _DEBUG
	if (traceModule) 
	{
		memset(name, 0, sizeof(name));
		length = cpiElementName(&rc, element, name, sizeof(name));
		completePrev = cpiElementCompletePrevious(&rc, element);
		completeNext = cpiElementCompleteNext(&rc, element);
		Log("\nPLUGIN: -> cpiParseLeftSibling() %S (0x%x) bits(%d %d)\n",
			name, element, completePrev, completeNext);
	}
#endif

	// Start the timer
	getStartTime(pc);

	// check for first time through
	if (pc->iOffset == 0)
	{
		validateMessage(pc);
	}

	// Check if we have a systems management message or a normal one
	if (pc->iMsgType == 0)
	{
		// Normal message, parse the next group until the element is completely parsed
		while ((cpiElementCompletePrevious(&rc, cpiParent(&rc, element)) != COMPLETE_BIT) &&
			   (cpiPreviousSibling(&rc, element) == CCI_NULL_ADDR) &&
			   (pc->iOffset < pc->iLength))
		{
			parseNextSegment(parser, pc);
		}

		/* add to the total time */
		time = calculateTime(pc);
		totalParseTime += time;
		pc->totalParseTime += time;
	}
	else
	{
		// system management message
		// handle as special case
		// This will build a complete parse tree
		parseMgmtMsg(parser, pc);
	}

#ifdef _DEBUG
	if (traceParser) 
	{
		completePrev = cpiElementCompletePrevious(&rc, element);
		completeNext = cpiElementCompleteNext(&rc, element);
		Log("PLUGIN: <- cpiParseLeftSibling() %S (0x%x) bits(%d %d)\n", 
			name, element, completePrev, completeNext);
	}
#endif

  return;
}

/******************************************************************************/
/*                                                                            */
/* Plugin Parser Implementation Function:                                     */
/*                                                                            */
/******************************************************************************/

void cpiParseRightSibling(CciParser* parser, CciContext* context, CciElement* element)
{
	PARSER_CONTEXT_ST*	pc = (PARSER_CONTEXT_ST*) context;
	int					rc;
	int					time;
#ifdef _DEBUG
	CciChar	name[256];
	CciSize	length;
	CciBool	completePrev;
	CciBool completeNext;
#endif

#ifdef _DEBUG
	if (traceModule > 0) 
	{
		memset(name, 0, sizeof(name));
		length = cpiElementName(&rc, element, name, sizeof(name));
		completePrev = cpiElementCompletePrevious(&rc, element);
		completeNext = cpiElementCompleteNext(&rc, element);
		Log("\nPLUGIN: -> cpiParseRightSibling() %S (0x%x) bits(%d %d)\n",
			name, element, completePrev, completeNext);
	}
#endif

	// Start the timer
	getStartTime(pc);

	// check for first time through
	if (pc->iOffset == 0)
	{
		validateMessage(pc);
	}

	// Check if we have a systems management message or a normal one
	if (pc->iMsgType == 0)
	{
		// Normal message, parse the next group until the element is completely parsed
		while ((cpiElementCompleteNext(&rc, cpiParent(&rc, element)) != COMPLETE_BIT) &&
			   (cpiNextSibling(&rc, element) == CCI_NULL_ADDR) &&
			   (pc->iOffset < pc->iLength))
		{
			parseNextSegment(parser, pc);
		}

		/* add to the total time */
		time = calculateTime(pc);
		totalParseTime += time;
		pc->totalParseTime += time;
	}
	else
	{
		// system management message
		// handle as special case
		// This will build a complete parse tree
		parseMgmtMsg(parser, pc);
	}

#ifdef _DEBUG
   if (traceModule > 0) 
   {
		completePrev = cpiElementCompletePrevious(&rc, element);
		completeNext = cpiElementCompleteNext(&rc, element);
		Log("PLUGIN: <- cpiParseRightSibling() %S (0x%x) bits(%d %d)\n", 
			name, element, completePrev, completeNext);
   }
#endif

	return;
}

/******************************************************************************/
/*                                                                            */
/* Plugin Parser Implementation Function:                                     */
/*                                                                            */
/******************************************************************************/

int cpiWriteBuffer(CciParser*  parser, CciContext*	context)

{
	PARSER_CONTEXT_ST* pc = (PARSER_CONTEXT_ST*)context;
	int				rc = 0;
	int				time;
	int				length;

#ifdef _DEBUG
	if (traceModule) 
	{
		Log("\nPLUGIN: -> cpiWriteBuffer() parser=0x%x context=0x%x\n", parser, context);
	}
#endif

	/* Reset the context area */
	resetContext(pc);

	// Start the timer
	getStartTime(pc);

	/* get the code page from the properties */
	pc->iCodePage = getCodePage(parser);

	/* make sure we have a sensible default code page */
	if (pc->iCodePage == 0)
	{
		pc->iCodePage = DEF_CODEPAGE;
	}

	/* get the numeric encoding from the properties for the message */
	pc->iEncoding = getEncoding(parser);

	/* make sure we have a proper encoding value */
	if (pc->iEncoding == 0)
	{
		/* set a sensible default */		pc->iEncoding = DEF_ENCODING;
	}

	/* create the output buffer from the logical message model */
	length = createOutputMessage(parser, pc);

	// keep track of the total bytes written
	totalWritten += length;

	// Calculate the response time
	time = calculateTime(pc);;

	if (time > maxWriteTime)
	{
		maxWriteTime = time;
	}

	if ((minWriteTime == 0) || (time < minWriteTime))
	{
		minWriteTime = time;
	}

	if (length > maxWriteLength)
	{
		maxWriteLength = length;
	}

	if ((minWriteLength == 0) || (length < minWriteLength))
	{
		minWriteLength = length;
	}

	// remember the time spent
	totalWriteTime += time;

	// Keep a count of the output messages
	writeCount++;

#ifdef _DEBUG
	if (traceModule) 
	{
		Log("PLUGIN: <- cpiWriteBuffer()\n");
	}
#endif

	return cpiBufferSize(&rc, parser);
}

/******************************************************************************/
/*                                                                            */
/* Plugin Parser Implementation Function:                                     */
/*                                                                            */
/******************************************************************************/

void cpiDeleteContext(CciParser*  parser,
					  CciContext* context)

{
	PARSER_CONTEXT_ST*	pc = (PARSER_CONTEXT_ST*)context;

#ifdef _DEBUG
	if (traceModule) 
	{
		Log("\nPLUGIN: -> cpiDeleteContext() parser=0x%x context=0x%x\n",
			parser, context);
	}
#endif

	if (pc->iBuffer != CCI_NULL_ADDR)
	{
		saveMsgStats(pc);
	}

	// free the buffer we acquired
	if (pc->iBuffer != CCI_NULL_ADDR)
	{
		free(pc->iBuffer);
	}

	// Free the context area
	free(pc);

	// Keep count of the number of parser objects
	contextCount--;

	if (contextCount == 0)
	{
		// Dump the statistics to a file
		dumpStatistics();
	}

#ifdef _DEBUG
	if (traceModule) 
	{
		Log("PLUGIN: <- cpiDeleteContext() count %d\n", contextCount);
	}
#endif

  return;
}

/******************************************************************************/
/*                                                                            */
/* Plugin Parser Implementation Function:                                     */
/*                                                                            */
/******************************************************************************/

CciBool cpiParserType(CciParser*  parser, CciContext* context)

{
	/* we are a body parser */
	return TRUE;
}

/******************************************************************************/
/*                                                                            */
/* Plugin Parser Implementation Function:                                     */
/*                                                                            */
/******************************************************************************/

void __declspec(dllexport) *bipGetParserFactory()
{
	/* Declare variables */
	CciFactory*     factoryObject;
	int             rc;
	static CPI_VFT  vftable;
  	SYSTEMTIME		curtime;
	FILETIME		filtime;

	// Get the current time
	GetSystemTime(&curtime);
	SystemTimeToFileTime(&curtime, &filtime);

	// Save the time the parser was started
	parserStartHigh = filtime.dwHighDateTime;
	parserStartLow  = filtime.dwLowDateTime;

	/* Setup function table with pointers to parser implementation functions */
	memset(&vftable, 0, sizeof(vftable));
	vftable.iFpCreateContext            = cpiCreateContext;
	vftable.iFpParseBuffer              = cpiParseBuffer;
	vftable.iFpParseFirstChild          = cpiParseFirstChild;
	vftable.iFpParseLastChild           = cpiParseLastChild;
	vftable.iFpParseLeftSibling         = cpiParseLeftSibling;
	vftable.iFpParseRightSibling        = cpiParseRightSibling;
	vftable.iFpWriteBuffer              = cpiWriteBuffer;
	vftable.iFpDeleteContext            = cpiDeleteContext;
	vftable.iFpIsHeaderParser           = cpiParserType;
	vftable.iFpSetValue                 = NULL;
	vftable.iFpValue                    = NULL;
	vftable.iFpNextParserClassName      = NULL;
	vftable.iFpSetNextParserClassName   = NULL;
	vftable.iFpNextParserEncoding       = NULL;
	vftable.iFpNextParserCodedCharSetId = NULL;

#ifdef _DEBUG
	/* check for DEBUG requested */
	initializeTrace();
#endif

	/* Initialize the file table for the meta data cache in memory */
	initializeMetadata();

	/* Create the parser factory for this plugin */
	factoryObject = cpiCreateParserFactory(&rc, ACORD_PARSER_NAME);
	if (factoryObject) 
	{
		/* Define the classes of message supported by the factory */
		cpiDefineParserClass(&rc, factoryObject, ACORD_PARSER_CLASS,  &vftable);
	}
	else 
	{
		/* Error: Unable to create parser factory */ 
#ifdef _DEBUG
		if (debug) 
		{
			Log("Unable to create parser factory, rc=%d\n", rc);
		}
#endif
	}

	/* Return address of this factory object to the broker */
	return(factoryObject);
}

const char * getSegParseLevel()

{
	return Level;
}

//////////////////////
// End of file
//////////////////////