//  miscsubs.cpp 

/*
 * Licensed Materials - Property of IBM
 * 5648-C63
 * (C) Copyright IBM Corp. 2000
 */

#include "miscsubs.h"
#include "trace.h"
#include "string.h"
#include "common.h"

#ifdef _DEBUG
extern int traceWrite;
extern int traceModule;
#endif

#ifdef _DEBUG
static char Level[]="miscsubs.cpp V1.0 Debug version ("__DATE__" "__TIME__")";
#else
static char Level[]="miscsubs.cpp V1.0 Release version ("__DATE__" "__TIME__")";
#endif

static const unsigned char HEX_NUMBERS[] = "0123456789ABCDEF";

int displayTruncated=0;			// Display fields truncated
int numericTruncated=0;			// Numeric fields truncated

///////////////////////////////////////////////////////////
//
// Convert memory bytes to hexadecimal representation
//
///////////////////////////////////////////////////////////

#ifdef _DEBUG
void bytes2hex(char * result, const char * input, int length)

{
	int i=0;
	int j=0;
	char ch;

	while (i < length)
	{
		ch = (unsigned char) input[i] >> 4;
		ch = HEX_NUMBERS[ch];
		result[j++] = ch;

		ch = (unsigned char) input[i] & 0x0F;
		ch = HEX_NUMBERS[ch];
		result[j++] = ch;

		i++;
	}

	// terminate the string
	result[j] = 0;
}
#endif
////////////////////////////////////////////////
//
// Set a value in an ASCII field
//
//
// Pad on the right with blanks
//
////////////////////////////////////////////////

void setDisplayValue(char *outBuf, int length, char * value, int valueLen)

{
	int	len;

#ifdef _DEBUG
	if (traceModule) 
	{
		Log("PLUGIN: -> setDisplayValue() outBuf=0x%x, value=%.10s, length=%d\n",
				outBuf, value, valueLen);
	}
#endif

	// make sure we don't have too much data
	// if we do, then truncate
	len = valueLen;
	if (length < valueLen)
	{
		len = length;

		// keep a count of the fields that are truncated
		displayTruncated++;
	}

	// move the data into the output field
	memcpy(outBuf, value, len);

	// pad with blanks on the right as needed
	while (len < length)
	{
		outBuf[len++] = ' ';
	}

#ifdef _DEBUG
	if (traceModule) 
	{
		Log("PLUGIN: <- setDisplayValue() value set outBuf=0x%x\n",
				outBuf);
	}
#endif
}

////////////////////////////////////////////////
//
// Set a value in a decimal field
//
// pad on the left with zeros
//
////////////////////////////////////////////////

void setNumericValue(char *outBuf, int length, char * value, int valueLen)

{
	int	len;
	int	i;

#ifdef _DEBUG
	if (traceModule) 
	{
		Log("PLUGIN: -> setNumericValue() outBuf=0x%x, value=%.10s, length=%d\n",
				outBuf, value, valueLen);
	}
#endif

	// make sure we don't have too much data
	// if we do, then truncate
	len = valueLen;
	if (length < valueLen)
	{
		len = length;

		// Keep a count of the fields truncated
		numericTruncated++;
	}

	// pad with zeros on the left as needed
	i = len;
	while (i < length)
	{
		outBuf[0] = '0';
		outBuf++;
		i++;
	}

	// move the data into the output field, starting
	// at the proper offset
	memcpy(outBuf, value, len);

#ifdef _DEBUG
	if (traceModule) 
	{
		Log("PLUGIN: <- setNumericValue() value set outBuf=0x%x\n", outBuf);
	}
#endif
}

const char * getMiscsubsLevel()

{
	return Level;
}