//  trace.cpp 

/*
 * Licensed Materials - Property of IBM
 * 5648-C63
 * (C) Copyright IBM Corp. 2000
 */

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <BipCci.h>
#include <BipCpi.h>
#include "trace.h"
#include "common.h"

#define ACORDTRACE			"ACORDTRACE"
#define ACORDTRACEFILE		"ACORDTRACEFILE"
#define ACORDTRACEPARSER	"ACORDTRACEPARSER"
#define ACORDTRACEWRITE		"ACORDTRACEWRITE"
#define ACORDTRACEMODULE	"ACORDTRACEMODULE"
#define ACORDTRACEMGMT		"ACORDTRACEMGMT"
#define ACORDTRACEFILEDEF	"C:\\acord\\Parser.trc"
#define TRACE_TYPE_LENGTH	8

#ifdef _DEBUG
static char Level[]="trace.cpp V1.0 debug version ("__DATE__" "__TIME__")";
#else
static char Level[]="trace.cpp V1.0 release version ("__DATE__" "__TIME__")";
#endif

#ifdef _DEBUG
static char		traceFile[256];
static FILE*	debugfile=NULL;		// Debug file stream 

int debug=1;
int traceParser=1;
int traceWrite=1;
int traceModule=1;
int traceMgmt=1;

static const unsigned char HEX_NUMBERS[] = "0123456789ABCDEF";

#endif

///////////////////////////////////////////////////////////
//
// Log errors to file
//
///////////////////////////////////////////////////////////

#ifdef _DEBUG
void Log(char *szFormat, ...)

{
	if (debug == 0)
	{
		return;
	}

	va_list	list;
	va_start(list, szFormat);

	if (debugfile != NULL)
	{
		vfprintf(debugfile, szFormat, list);
		fflush(debugfile);
		va_end(list);
	}
}
#endif

///////////////////////////////////////////////////////////
//
// Dump memory to debug file
//
///////////////////////////////////////////////////////////

#ifdef _DEBUG
void dumpMemory(unsigned char *data, int length)

{
	int		i;
	int		j;			// work variable
	int		k;			// work variable
	char	hexStr[256];
	char	asciiStr[256];
	char	ch;			// work variable
	int		buffer=0;
	int		remaining;	// number of bytes left to process
	int		offset;		// current location within the raw file data
	int		count;		// number of lines
	int		location=0; // offset of start of this line within the file

	if (debug == 0)
	{
		return;
	}

	remaining = length;

	Log("dumping memory at 0x%x for %d bytes\n", data, length);

	while (remaining > 0)
	{
		// add the offset to the beginning of the line
		sprintf(asciiStr, "%4.4d ", location);

		location += 32;

		if (remaining > 32)
		{
			count = 32;
		}
		else
		{
			count = remaining;
		} // endif 

		// initialize the output strings and buffer offset 
		offset = buffer;
		i = 5;    // initialize variable offset
		j = 0;    // initial variable offset
		k = 0;    // initialize counter 

		while (k < count)
		{
			if ((data[buffer] >= ' ') &&
				((unsigned char) data[buffer] <= 127))
			{
				asciiStr[i++] = data[buffer];
			}
			else
			{
				asciiStr[i++] = '.';
			} // endif 

			ch = (unsigned char) data[buffer] >> 4;
			ch = HEX_NUMBERS[ch];
			hexStr[j++] = ch;

			ch = (unsigned char) data[buffer] & 0x0F;
			ch = HEX_NUMBERS[ch];
			hexStr[j++] = ch;

			// Go on to the next character 
			remaining--;
			buffer++;
			k++;   // increment counter 

			// Insert a blank for every fourth character in hex string 
			if (k%4 == 0)
			{
				hexStr[j++] = ' ';
			} // endif 

			// Insert a blank for every eighth character in ASCII string 
			if (k%8 == 0)
			{
				asciiStr[i++] = ' ';
			} // endif 
		} // endwhile 

		// pad the ascii string with blanks to proper length
		while (i < 41)
		{
			asciiStr[i++] = ' ';
		}

		asciiStr[i] = 0;		// Terminate the strings
		hexStr[j] = 0;

		Log("%s %s\n", asciiStr, hexStr);
	} // endwhile 
}
#endif

////////////////////////////////////////////////
//
// Dump an area of data to the debug file
//
////////////////////////////////////////////////

#ifdef _DEBUG
void dumpElement (CciElement *element)

{
	unsigned char* ptr;
	unsigned char* ptr2;

	ptr = (unsigned char*) element;

	dumpMemory(ptr, 112);
	ptr2 = (unsigned char*) (*ptr);
	dumpMemory(ptr2, 32);
	ptr2 += 12;
	dumpMemory(ptr2, 32);
}
#endif

#ifdef _DEBUG
void startTrace()

{
	if (debug >= 1)
	{
		debugfile = fopen(traceFile, "a+");
		Log("Starting trace!!! - Acord Parser level 1.00 %s %s\n", __DATE__, __TIME__);
		Log("Trace settings are debug=%d, Parser=%d, Writes=%d, Modules=%d, Mgmt=%d\n",
			debug, traceParser, traceWrite, traceModule, traceMgmt);
	}
}

void setTraceFile()

{
	if (debugfile == NULL)
	{
		/* Do we need to open the trace file? */
		if (debug >= 1)
		{
			startTrace();
			Log("Trace on request processed!!!\n");
		}
	}
	else
	{
		/* Should we close the trace file? */
		if (debug == 0)
		{
			/* record what we are doing in the trace file */
			fprintf(debugfile, "Trace off request processed!!!\n");
			fprintf(debugfile, "Tracing stopped!!!\n\n");
			fflush(debugfile);

			/* close the trace file */
			fclose(debugfile);
			debugfile = NULL;
		}
	}
}
#endif

void initializeTrace()

{
#ifdef _DEBUG
	int		traceVal;
	char*	traceVar;

	/* check for trace setting in environment */
	traceVar = getenv(ACORDTRACE);
	if (traceVar != NULL)
	{
		traceVal = atoi(traceVar);
		if (traceVal > 0)
		{
			debug = traceVal;
		}
	}

	/* check for trace setting in environment */
	traceVar = getenv(ACORDTRACEMGMT);
	if (traceVar != NULL)
	{
		traceVal = atoi(traceVar);
		if (traceVal > 0)
		{
			traceMgmt = traceVal;
		}
	}

	/* check for trace setting in environment */
	traceVar = getenv(ACORDTRACEMODULE);
	if (traceVar != NULL)
	{
		traceVal = atoi(traceVar);
		if (traceVal > 0)
		{
			traceModule = traceVal;
		}
	}

	/* check for trace setting in environment */
	traceVar = getenv(ACORDTRACEPARSER);
	if (traceVar != NULL)
	{
		traceVal = atoi(traceVar);
		if (traceVal > 0)
		{
			traceParser = traceVal;
		}
	}

	/* check for trace setting in environment */
	traceVar = getenv(ACORDTRACEWRITE);
	if (traceVar != NULL)
	{
		traceVal = atoi(traceVar);
		if (traceVal > 0)
		{
			traceWrite = traceVal;
		}
	}

	/* see if there is a trace file name set in an environment variable */
	memset(traceFile, 0, sizeof(traceFile));
	traceVar = getenv(ACORDTRACEFILE);
	if (traceVar == NULL)
	{
		/* no environment variable, take default */
		strcpy(traceFile, ACORDTRACEFILEDEF);
	}
	else
	{
		/* use the environment variable */
		strncpy(traceFile, traceVar, sizeof(traceFile) - 1);
	}

	startTrace();
#endif
}

/******************************************************************************/
/*                                                                            */
/* Routine to process a request to turn tracing on or off.                 .  */
/*                                                                            */
/******************************************************************************/

void processTraceMessage(char * traceTypes, int count, int setting)

{
#ifdef _DEBUG
	int	i;
	char* ptr;

	if (traceModule)
	{
		Log("PLUGIN: -> processTraceMessage(): count=%d, setting=%d\n", count, setting);
	}

	ptr = traceTypes;
	i = 0;
	while (i < count)
	{
		if ((memcmp(ptr, TRACEALL, sizeof(TRACEALL) - 1) == 0) ||
			(memcmp(ptr, TRACEOFF, sizeof(TRACEOFF) - 1) == 0))
		{
			traceParser = setting;
			traceWrite = setting;
			traceModule = setting;
			traceMgmt = setting;
		}

		if (memcmp(ptr, TRACEPARSER, sizeof(TRACEPARSER) - 1) == 0)
		{
			traceParser = setting;
		}

		if (memcmp(ptr, TRACEWRITE, sizeof(TRACEWRITE) - 1) == 0)
		{
			traceWrite = setting;
		}

		if (memcmp(ptr, TRACEMODULE, sizeof(TRACEMODULE) - 1) == 0)
		{
			traceModule = setting;
		}

		if (memcmp(ptr, TRACEMGMT, sizeof(TRACEMGMT) - 1) == 0)
		{
			traceMgmt = setting;
		}

		ptr += TRACE_TYPE_LENGTH;
		i++;
	}

	/* set the overall trace status */
	if ((traceParser == 1) || (traceWrite == 1) || (traceModule == 1) || (traceMgmt == 1))
	{
		debug = 1;
	}
	else
	{
		debug = 0;
	}

	setTraceFile();

	if (traceModule)
	{
		Log("PLUGIN: <- processTraceMessage(): debug=%d, traceModule=%d traceParser=%d traceWrite=%d traceMgmt=%d\n", 
			debug, traceModule, traceParser, traceWrite, traceMgmt);
	}
#endif
}

//////////////////////////////////////////////
//
// Routine to dump out the values of a
// particular element
//
//////////////////////////////////////////////

#ifdef _DEBUG
void dumpElement(CciElement * element, const int indent)

{
	int				rc;
	int				i;
	CciElementType	type;
	CciSize			size;
	char			spaces[32];
	CciChar			name[512];
	CciChar			value[512];
	CciBool			completePrev;
	CciBool			completeNext;

	// get a string with the right number of spaces for indenting the line
	spaces[0] = 0;
	i = 0;
	while ((i < indent) && (i < (sizeof(spaces) - 1)))
	{
		strcat(spaces, " ");
		i++;
	}

	type = cpiElementType(&rc, element);
	completePrev = cpiElementCompletePrevious(&rc, element);
	completeNext = cpiElementCompleteNext(&rc, element);
	size = cpiElementName(&rc, element, name, sizeof(name) - 1);

	if (type == CCI_ELEMENT_TYPE_NAME)
	{
		Log ("%s name element %S bits(%d %d)\n", spaces, name, completePrev, completeNext);
	}
	else
	{
		size = cpiElementCharacterValue(&rc, element, value, sizeof(value) - 1);
		Log("%s element %S bits(%d %d) value %S\n", spaces, name, completePrev, completeNext, value);
	}
}
#endif

///////////////////////////////////////////////
//
// Routine to dump out all the elements under
// the specified element, including the name,
// element address and completion bits.
//
///////////////////////////////////////////////

#ifdef _DEBUG
void dumpAllElements(CciElement * root, const int indent)

{
	int				rc;
	CciElement*		element;

	// dump this element to the trace log
	dumpElement(root, indent);

	// initialize the element to the first child of the root
	element = cpiFirstChild(&rc, root);
	while ((rc == CCI_SUCCESS) && (element != CCI_NULL_ADDR))
	{
		dumpAllElements(element, indent + 1);

		// move on to the next sibling at this level
		element = cpiNextSibling(&rc, element);
	}
}
#endif

const char * getTraceLevel()

{
	return Level;
}